# frozen_string_literal: true

#-------------------------------------------------------------------------
# # Copyright (c) Microsoft and contributors. All rights reserved.
#
# The MIT License(MIT)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#--------------------------------------------------------------------------
require "cgi"
require "azure/core/auth/signer"
require "azure/core/auth/shared_key"

module Azure::Storage::Common::Core
  module Auth
    class SharedKey < Azure::Core::Auth::SharedKey
      # Generate the string to sign.
      #
      # @param method     [Symbol] HTTP request method.
      # @param uri        [URI] URI of the request we're signing.
      # @param headers    [Hash] HTTP request headers.
      #
      # @return [String]
      def signable_string(method, uri, headers)
        [
          method.to_s.upcase,
          headers.fetch("Content-Encoding", ""),
          headers.fetch("Content-Language", ""),
          headers.fetch("Content-Length", "").sub(/^0+/, ""), # from 2015-02-21, if Content-Length == 0, it won't be signed
          headers.fetch("Content-MD5", ""),
          headers.fetch("Content-Type", ""),
          headers.fetch("Date", ""),
          headers.fetch("If-Modified-Since", ""),
          headers.fetch("If-Match", ""),
          headers.fetch("If-None-Match", ""),
          headers.fetch("If-Unmodified-Since", ""),
          headers.fetch("Range", ""),
          canonicalized_headers(headers),
          canonicalized_resource(uri)
        ].join("\n")
      end
    end
  end
end
